// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KWAYLAND_VERSION_H
#define KWAYLAND_VERSION_H

#define KWAYLAND_VERSION_STRING "6.6.4"
#define KWAYLAND_VERSION_MAJOR 6
#define KWAYLAND_VERSION_MINOR 6
#define KWAYLAND_VERSION_PATCH 4
#define KWAYLAND_VERSION ((6<<16)|(6<<8)|(4))

#endif
